import type {
  DailyResult,
  ElecCostSummary,
  DailyEntry,
  StandEntry,
  ElecStandCostSummary,
  StandResult,
} from "../utils/types";
import useElecPriceStore from "../stores/useElecPriceStore";
import useElecDemandStore from "../stores/useElecDemandStore";

const storeElecPrice = useElecPriceStore();
const ContractUseValue = useElecDemandStore();

export const CalcuEleCost = (input: Map<string, number>): ElecCostSummary => {
  const dailyData: Map<string, DailyEntry[]> = new Map();
  let totalFlowCost = 0; // 總電價
  let totalEleCost = 0; //總用電
  let total_Off = 0; //總離峰用電
  let total_half = 0; //總半尖峰用電
  let total_peak = 0; //總尖峰用電
  let total_OffCost = 0; //總離峰電價
  let total_halfCost = 0; //總半尖峰電價
  let total_peakCost = 0; //總尖峰電價

  const dailyResults: DailyResult[] = [];
  const elecPrices = storeElecPrice.elecData;
  const Summer_Off_Prices =
    elecPrices.find((item: any) => item.displayName === "流動週日離峰夏月")
      ?.out || 0;
  const Summer_HalfPeak_Prices_Saturday =
    elecPrices.find((item: any) => item.displayName === "流動週六半尖峰夏月")
      ?.out || 0;
  const Summer_HalfPeak_Prices_Weekday =
    elecPrices.find((item: any) => item.displayName === "流動平日半尖峰夏月")
      ?.out || 0;
  const Summer_Peak_Prices =
    elecPrices.find((item: any) => item.displayName === "流動平日尖峰夏月")
      ?.out || 0;
  const Non_Summer_Off_Prices =
    elecPrices.find((item: any) => item.displayName === "流動週日離峰非夏月")
      ?.out || 0;
  const Non_Summer_HalfPeak_Prices_Saturday =
    elecPrices.find((item: any) => item.displayName === "流動週六半尖峰非夏月")
      ?.out || 0;
  const Non_Summer_HalfPeak_Prices_Weekday =
    elecPrices.find((item: any) => item.displayName === "流動平日半尖峰非夏月")
      ?.out || 0;
  // 1. 將輸入資料按日期分組
  input.forEach((value, key) => {
    const dateStr = key.substring(0, 10);
    if (!dailyData.has(dateStr)) {
      dailyData.set(dateStr, []);
    }
    dailyData.get(dateStr)?.push({ time: new Date(key), value });
  });

  for (const [dateStr, entries] of dailyData.entries()) {
    if (!entries || entries.length === 0) continue;

    const sampleDate = entries[0].time;
    const dayOfWeek = sampleDate.getDay();
    const year = sampleDate.getFullYear();
    const summerStart = new Date(`${year}-05-16`);
    const summerEnd = new Date(`${year}-10-15`);

    let off = 0; //離峰用電
    let half = 0; //半尖峰用電
    let peak = 0; //尖峰用電
    let offcost = 0; //離峰電價
    let halfcost = 0; //半尖峰電價
    let peakcost = 0; //尖峰電價
    let dailyFlowCost = 0; //當日電價
    let dailyEleCost = 0; //當日用電

    const isSummer = sampleDate >= summerStart && sampleDate <= summerEnd;

    entries.forEach(({ time, value }) => {
      const hour = time.getHours();

      if (isSummer) {
        if (dayOfWeek === 0) {
          off += value;
        } else if (dayOfWeek === 6) {
          if (hour < 9) off += value;
          else half += value;
        } else {
          if (hour >= 16 && hour < 22) peak += value;
          else if ((hour >= 9 && hour < 16) || hour >= 22) half += value;
          else off += value;
        }
      } else {
        if (dayOfWeek === 0) {
          off += value;
        } else {
          if (hour < 6 || (hour >= 11 && hour < 14)) off += value;
          else half += value;
        }
      }
    });

    if (isSummer) {
      console.log("夏月計費");
      const summerHalfPeakRate =
        dayOfWeek === 6
          ? Summer_HalfPeak_Prices_Saturday
          : Summer_HalfPeak_Prices_Weekday;
      offcost = off * Summer_Off_Prices;
      halfcost = half * summerHalfPeakRate;
      peakcost = peak * Summer_Peak_Prices;
      dailyFlowCost = offcost + halfcost + peakcost;
      dailyEleCost = off + half + peak;
    } else {
      console.log("非夏月計費");
      const nonSummerHalfPeakRate =
        dayOfWeek === 6
          ? Non_Summer_HalfPeak_Prices_Saturday
          : Non_Summer_HalfPeak_Prices_Weekday;
      offcost = off * Non_Summer_Off_Prices;
      halfcost = half * nonSummerHalfPeakRate;
      dailyFlowCost = offcost + halfcost;
      dailyEleCost = off + half;
    }

    totalFlowCost += dailyFlowCost;
    totalEleCost += dailyEleCost;
    total_Off += off;
    total_half += half;
    total_peak += peak;
    total_OffCost += offcost;
    total_halfCost += halfcost;
    total_peakCost += peakcost;
    dailyResults.push({
      dateStr,
      off,
      half,
      peak,
      offcost,
      halfcost,
      peakcost,
      dailyEleCost,
      dailyFlowCost,
    });
  }
  return {
    dailyResults,
    totalEleCost,
    totalFlowCost,
    total_Off,
    total_half,
    total_peak,
    total_OffCost,
    total_halfCost,
    total_peakCost,
  };
};

export const CalcuEleStandCost = (
  input: Map<string, number>
): ElecStandCostSummary => {
  const elecPrices = storeElecPrice.elecData;
  const ContractUseData = ContractUseValue.elecData;

  const Three_Phase =
    elecPrices.find((item: any) => item.displayName === "基本按戶三相")?.out ||
    0;
  const Summer_Regular_Use =
    elecPrices.find((item: any) => item.displayName === "基本經常夏月")?.out ||
    0;
  const Non_Summer_Regular_Use =
    elecPrices.find((item: any) => item.displayName === "基本經常非夏月")
      ?.out || 0;
  const ContractUse =
    ContractUseData.find((item: any) => item.name === "Engel")?.out || 0;

  const monthMap = new Map<string, StandEntry[]>(); // key: yyyy-MM

  // 將資料依據年月分組
  input.forEach((value, key) => {
    const date = new Date(key);
    const year = date.getFullYear();
    const month = date.getMonth() + 1;
    const ymKey = `${year}-${month.toString().padStart(2, "0")}`;

    if (!monthMap.has(ymKey)) {
      monthMap.set(ymKey, []);
    }
    monthMap.get(ymKey)?.push({ time: date, value });
  });

  const StandResults: StandResult[] = [];
  let totalStandCost = 0;

  for (const [ym, entries] of monthMap.entries()) {
    if (!entries || entries.length === 0) continue;
    
    const sampleDate = entries[0].time;
    //const month = sampleDate.getMonth() + 1;
    // 修改 isSummer 判斷方式：從月改為日期範圍 5/16 ~ 10/15
    const year = sampleDate.getFullYear();
    const summerStart = new Date(`${year}-05-16`);
    const summerEnd = new Date(`${year}-10-15`);
    const isSummer = sampleDate >= summerStart && sampleDate <= summerEnd;

    let Phase = Three_Phase;
    let Contract = isSummer ? Summer_Regular_Use : Non_Summer_Regular_Use;

    const StandCost = Phase + Contract * ContractUse;

    StandResults.push({
      Phase,
      Contract,
      ContractUse,
      StandCost,
    });

    totalStandCost += StandCost;
    console.log(`=== ${ym} ===`);
    console.log(`  按戶類別: ${Phase.toFixed(2)}`);
    console.log(`  契約類型: $${Contract.toFixed(2)}`);
    console.log(`  契約度數: $${ContractUse.toFixed(2)}`);
    console.log(`  基本電價: $${StandCost.toFixed(2)}`);
  }

  return {
    StandResults,
    StandCost: totalStandCost,
  };
};
